@@ -1,7 +1,7 @@
 # Automated tests for the `coloredlogs' package.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: February 14, 2020
+# Last Change: February 15, 2020
 # URL: https://coloredlogs.readthedocs.io
 
 """Automated tests for the `coloredlogs` package."""
@@ -20,7 +20,7 @@ import tempfile
 # External dependencies.
 from humanfriendly.compat import StringIO
 from humanfriendly.terminal import ANSI_COLOR_CODES, ansi_style, ansi_wrap
-from humanfriendly.testing import PatchedItem, TestCase, retry
+from humanfriendly.testing import PatchedAttribute, PatchedItem, TestCase, retry
 from humanfriendly.text import format, random_string
 from mock import MagicMock
 
@@ -46,6 +46,7 @@ from coloredlogs import (
     set_level,
     walk_propagation_tree,
 )
+from coloredlogs.demo import demonstrate_colored_logging
 from coloredlogs.syslog import SystemLogging, match_syslog_handler
 from coloredlogs.converter import (
     ColoredCronMailer,
@@ -415,42 +416,42 @@ class ColoredLogsTestCase(TestCase):
                 '<code>plain text followed by <span style="color:{css}">{name}</span> text</code>',
                 css=EIGHT_COLOR_PALETTE[ansi_code], name=color_name,
             )
-            self.assertEquals(expected_html, convert(ansi_encoded_text))
+            self.assertEqual(expected_html, convert(ansi_encoded_text))
         # Check conversion of bright colored text.
         expected_html = '<code><span style="color:#FF0">bright yellow</span></code>'
-        self.assertEquals(expected_html, convert(ansi_wrap('bright yellow', color='yellow', bright=True)))
+        self.assertEqual(expected_html, convert(ansi_wrap('bright yellow', color='yellow', bright=True)))
         # Check conversion of text with a background color.
         expected_html = '<code><span style="background-color:#DE382B">red background</span></code>'
-        self.assertEquals(expected_html, convert(ansi_wrap('red background', background='red')))
+        self.assertEqual(expected_html, convert(ansi_wrap('red background', background='red')))
         # Check conversion of text with a bright background color.
         expected_html = '<code><span style="background-color:#F00">bright red background</span></code>'
-        self.assertEquals(expected_html, convert(ansi_wrap('bright red background', background='red', bright=True)))
+        self.assertEqual(expected_html, convert(ansi_wrap('bright red background', background='red', bright=True)))
         # Check conversion of text that uses the 256 color mode palette as a foreground color.
         expected_html = '<code><span style="color:#FFAF00">256 color mode foreground</span></code>'
-        self.assertEquals(expected_html, convert(ansi_wrap('256 color mode foreground', color=214)))
+        self.assertEqual(expected_html, convert(ansi_wrap('256 color mode foreground', color=214)))
         # Check conversion of text that uses the 256 color mode palette as a background color.
         expected_html = '<code><span style="background-color:#AF0000">256 color mode background</span></code>'
-        self.assertEquals(expected_html, convert(ansi_wrap('256 color mode background', background=124)))
+        self.assertEqual(expected_html, convert(ansi_wrap('256 color mode background', background=124)))
         # Check that invalid 256 color mode indexes don't raise exceptions.
         expected_html = '<code>plain text expected</code>'
-        self.assertEquals(expected_html, convert('\x1b[38;5;256mplain text expected\x1b[0m'))
+        self.assertEqual(expected_html, convert('\x1b[38;5;256mplain text expected\x1b[0m'))
         # Check conversion of bold text.
         expected_html = '<code><span style="font-weight:bold">bold text</span></code>'
-        self.assertEquals(expected_html, convert(ansi_wrap('bold text', bold=True)))
+        self.assertEqual(expected_html, convert(ansi_wrap('bold text', bold=True)))
         # Check conversion of underlined text.
         expected_html = '<code><span style="text-decoration:underline">underlined text</span></code>'
-        self.assertEquals(expected_html, convert(ansi_wrap('underlined text', underline=True)))
+        self.assertEqual(expected_html, convert(ansi_wrap('underlined text', underline=True)))
         # Check conversion of strike-through text.
         expected_html = '<code><span style="text-decoration:line-through">strike-through text</span></code>'
-        self.assertEquals(expected_html, convert(ansi_wrap('strike-through text', strike_through=True)))
+        self.assertEqual(expected_html, convert(ansi_wrap('strike-through text', strike_through=True)))
         # Check conversion of inverse text.
         expected_html = '<code><span style="background-color:#FFC706;color:#000">inverse</span></code>'
-        self.assertEquals(expected_html, convert(ansi_wrap('inverse', color='yellow', inverse=True)))
+        self.assertEqual(expected_html, convert(ansi_wrap('inverse', color='yellow', inverse=True)))
         # Check conversion of URLs.
         for sample_text in 'www.python.org', 'http://coloredlogs.rtfd.org', 'https://coloredlogs.rtfd.org':
             sample_url = sample_text if '://' in sample_text else ('http://' + sample_text)
             expected_html = '<code><a href="%s" style="color:inherit">%s</a></code>' % (sample_url, sample_text)
-            self.assertEquals(expected_html, convert(sample_text))
+            self.assertEqual(expected_html, convert(sample_text))
         # Check that the capture pattern for URLs doesn't match ANSI escape
         # sequences and also check that the short hand for the 0 reset code is
         # supported. These are tests for regressions of bugs found in
@@ -464,7 +465,7 @@ class ColoredLogsTestCase(TestCase):
             'https://coloredlogs.readthedocs.io'
             '</a></span>&gt;</code>'
         )
-        self.assertEquals(expected_html, convert(ansi_encoded_text))
+        self.assertEqual(expected_html, convert(ansi_encoded_text))
 
     def test_output_interception(self):
         """Test capturing of output from external commands."""
@@ -538,6 +539,44 @@ class ColoredLogsTestCase(TestCase):
         """Test that the usage message is shown when ``--help`` is given."""
         assert 'Usage:' in main('coloredlogs', '--help', capture=True)
 
+    def test_custom_record_factory(self):
+        """
+        Test that custom LogRecord factories are supported.
+
+        This test is a bit convoluted because the logging module suppresses
+        exceptions. We monkey patch the method suspected of encountering
+        exceptions so that we can tell after it was called whether any
+        exceptions occurred (despite the exceptions not propagating).
+        """
+        if not hasattr(logging, 'getLogRecordFactory'):
+            return self.skipTest("this test requires Python >= 3.2")
+
+        exceptions = []
+        original_method = ColoredFormatter.format
+        original_factory = logging.getLogRecordFactory()
+
+        def custom_factory(*args, **kwargs):
+            record = original_factory(*args, **kwargs)
+            record.custom_attribute = 0xdecafbad
+            return record
+
+        def custom_method(*args, **kw):
+            try:
+                return original_method(*args, **kw)
+            except Exception as e:
+                exceptions.append(e)
+                raise
+
+        with PatchedAttribute(ColoredFormatter, 'format', custom_method):
+            logging.setLogRecordFactory(custom_factory)
+            try:
+                demonstrate_colored_logging()
+            finally:
+                logging.setLogRecordFactory(original_factory)
+
+        # Ensure that no exceptions were triggered.
+        assert not exceptions
+
 
 def check_contents(filename, contents, match):
     """Check if a line in a file contains an expected string."""
